@@ -219,6 +219,7 @@ static void configure_handler_event_types(short event_types);
 static uint16_t *compressed_dname_offsets = 0;
 static uint32_t compression_table_capacity = 0;
 static uint32_t compression_table_size = 0;
+static domain_type* compressed_dnames[MAXRRSPP];
 
 /*
  * Remove the specified pid from the list of child pids.  Returns -1 if
@@ -308,6 +309,9 @@ restart_child_servers(struct nsd *nsd, region_type* region, netio_type* netio,
 				/* the child need not be able to access the
 				 * nsd.db file */
 				namedb_close_udb(nsd->db);
+#ifdef MEMCLEAN /* OS collects memory pages */
+				region_destroy(region);
+#endif
 				nsd->pid = 0;
 				nsd->child_count = 0;
 				nsd->server_kind = nsd->children[i].kind;
@@ -561,7 +565,7 @@ server_init_ifs(struct nsd *nsd, size_t from, size_t to, int* reuseport_works)
 {
 	struct addrinfo* addr;
 	size_t i;
-#if defined(SO_REUSEPORT) || defined(SO_REUSEADDR) || (defined(INET6) && (defined(IPV6_V6ONLY) || defined(IPV6_USE_MIN_MTU) || defined(IPV6_MTU) || defined(IP_TRANSPARENT)) || defined(IP_FREEBIND))
+#if defined(SO_REUSEPORT) || defined(SO_REUSEADDR) || (defined(INET6) && (defined(IPV6_V6ONLY) || defined(IPV6_USE_MIN_MTU) || defined(IPV6_MTU) || defined(IP_TRANSPARENT)) || defined(IP_FREEBIND) || defined(SO_BINDANY))
 	int on = 1;
 #endif
 
@@ -751,6 +755,12 @@ server_init_ifs(struct nsd *nsd, size_t from, size_t to, int* reuseport_works)
 					strerror(errno));
 			}
 #endif /* IP_TRANSPARENT */
+#ifdef SO_BINDANY
+			if (setsockopt(nsd->udp[i].s, SOL_SOCKET, SO_BINDANY, &on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(...,SO_BINDANY, ...) failed for udp: %s",
+					strerror(errno));
+			}
+#endif /* SO_BINDANY */
 		}
 
 		if (bind(nsd->udp[i].s, (struct sockaddr *) addr->ai_addr, addr->ai_addrlen) != 0) {
@@ -881,6 +891,12 @@ server_init_ifs(struct nsd *nsd, size_t from, size_t to, int* reuseport_works)
 					strerror(errno));
 			}
 #endif /* IP_TRANSPARENT */
+#ifdef SO_BINDANY
+			if (setsockopt(nsd->tcp[i].s, SOL_SOCKET, SO_BINDANY, &on, sizeof(on)) < 0) {
+				log_msg(LOG_ERR, "setsockopt(...,SO_BINDANY, ...) failed for tcp: %s",
+					strerror(errno));
+			}
+#endif /* SO_BINDANY */
 		}
 
 		if (bind(nsd->tcp[i].s, (struct sockaddr *) addr->ai_addr, addr->ai_addrlen) != 0) {
@@ -1061,7 +1077,14 @@ server_shutdown(struct nsd *nsd)
 	daemon_remote_delete(nsd->rc); /* ssl-delete secret keys */
 #endif
 
-#if 0 /* OS collects memory pages */
+#ifdef MEMCLEAN /* OS collects memory pages */
+#ifdef RATELIMIT
+	rrl_mmap_deinit_keep_mmap();
+#endif
+	udb_base_free_keep_mmap(nsd->task[0]);
+	udb_base_free_keep_mmap(nsd->task[1]);
+	namedb_close_udb(nsd->db); /* keeps mmap */
+	namedb_close(nsd->db);
 	nsd_options_destroy(nsd->options);
 	region_destroy(nsd->region);
 #endif
@@ -1829,7 +1852,7 @@ server_main(struct nsd *nsd)
 			/* only quit children after xfrd has acked */
 			send_children_quit(nsd);
 
-#if 0 /* OS collects memory pages */
+#ifdef MEMCLEAN /* OS collects memory pages */
 			region_destroy(server_region);
 #endif
 			server_shutdown(nsd);
@@ -1908,7 +1931,7 @@ server_main(struct nsd *nsd)
 		(void)kill(nsd->pid, SIGTERM);
 	}
 
-#if 0 /* OS collects memory pages */
+#ifdef MEMCLEAN /* OS collects memory pages */
 	region_destroy(server_region);
 #endif
 	/* write the nsd.db to disk, wait for it to complete */
@@ -1978,6 +2001,8 @@ server_child(struct nsd *nsd)
 		log_msg(LOG_ERR, "nsd server could not create event base");
 		exit(1);
 	}
+	nsd->event_base = event_base;
+	nsd->server_region = server_region;
 
 #ifdef RATELIMIT
 	rrl_init(nsd->this_child->child_num);
@@ -2026,13 +2051,15 @@ server_child(struct nsd *nsd)
 	if (nsd->server_kind & NSD_SERVER_UDP) {
 #if (defined(NONBLOCKING_IS_BROKEN) || !defined(HAVE_RECVMMSG))
 		udp_query = query_create(server_region,
-			compressed_dname_offsets, compression_table_size);
+			compressed_dname_offsets, compression_table_size,
+			compressed_dnames);
 #else
 		udp_query = NULL;
 		memset(msgs, 0, sizeof(msgs));
 		for (i = 0; i < NUM_RECV_PER_SELECT; i++) {
 			queries[i] = query_create(server_region,
-				compressed_dname_offsets, compression_table_size);
+				compressed_dname_offsets,
+				compression_table_size, compressed_dnames);
 			query_reset(queries[i], UDP_MAX_MESSAGE_LEN, 0);
 			iovecs[i].iov_base          = buffer_begin(queries[i]->packet);
 			iovecs[i].iov_len           = buffer_remaining(queries[i]->packet);;
@@ -2144,7 +2171,10 @@ server_child(struct nsd *nsd)
 	bind8_stats(nsd);
 #endif /* BIND8_STATS */
 
-#if 0 /* OS collects memory pages */
+#ifdef MEMCLEAN /* OS collects memory pages */
+#ifdef RATELIMIT
+	rrl_deinit(nsd->this_child->child_num);
+#endif
 	event_base_free(event_base);
 	region_destroy(server_region);
 #endif
@@ -2902,7 +2932,7 @@ handle_tcp_accept(int fd, short event, void* arg)
 		tcp_region, sizeof(struct tcp_handler_data));
 	tcp_data->region = tcp_region;
 	tcp_data->query = query_create(tcp_region, compressed_dname_offsets,
-		compression_table_size);
+		compression_table_size, compressed_dnames);
 	tcp_data->nsd = data->nsd;
 	tcp_data->query_count = 0;
 
